import numpy as np 
qubic_solids_26 = ['Li-bcc','Na-bcc','K-bcc','Rb-bcc',
                   'Ca-fcc','Sr-fcc','Ba-bcc',
                   'Nb-bcc','Ta-bcc',
                   'Mo-bcc','W-bcc','Fe-bcc',
                   'Rh-fcc','Ir-fcc',
                   'Ni-fcc','Pd-fcc','Pt-fcc',
                   'Cu-fcc','Ag-fcc','Au-fcc',
                   'Al-fcc','Pb-fcc',              #KRIS: WHY WERE Pb-fcc and Au-fcc commented out???
                   'C-diamond','Si-diamond','Ge-diamond','Sn-diamond']
qubic_solids_27 = np.append(qubic_solids_26, 'V-bcc')

hcp_solids_10 = ['Cd-hcp', 'Co-hcp', 'Os-hcp', 'Ru-hcp', 'Zn-hcp',
                 'Zr-hcp', 'Sc-hcp', 'Be-hcp', 'Mg-hcp', 'Ti-hcp']


# ZPAE (semi-empirical) corrected lattice parameters from Haas, Tran, Blaha, PRB 79, 085104 (2009)
# and ZPAE (phonon) corrected lattice parameters + Debye temps + bulk moduli (GPa) from Hao et al., PRB 85, 014111 (2012)
# cohesive energies (at 0 K) and Debye temperatures from Kittel and Hao et al.
data = {
# Cubic pure solids
'Li-bcc': {
    'name': 'Li-bcc',
    'lattice parameter': 3.451,
    'pbe-lp': 3.429,
    'cohesive energy': 1.63,
    'debye temperature': 344.,
    'bulk modulus': 13.,
    'symbols': 'Li',
    'latex name': "Li",
    'structure':'bcc',
    'magmom': None},
'Na-bcc': {
    'name': 'Na-bcc',
    'lattice parameter': 4.207,
    'pbe-lp': 4.197,
    'cohesive energy': 1.113,
    'bulk modulus': 7.5,
    'debye temperature': 158.,
    'symbols': 'Na',
    'latex name': "Na",
    'structure':'bcc',
    'magmom': None},
'K-bcc': {
    'name': 'K-bcc',
    'lattice parameter': 5.211,
    'pbe-lp': 5.281,
    'cohesive energy': 0.934,
    'bulk modulus': 3.7,
    'debye temperature': 91.,
    'symbols': 'K',
    'latex name': "K",
    'structure':'bcc',
    'magmom': None},
'Rb-bcc': {
    'name': 'Rb-bcc',
    'lattice parameter': 5.580,
    'pbe-lp': 5.665,
    'cohesive energy': 0.852,
    'bulk modulus': 3.1,
    'debye temperature': 56.,
    'symbols': 'Rb',
    'latex name': "Rb",
    'structure':'bcc',
    'magmom': None},
'Ca-fcc': {
    'name': 'Ca-fcc',
    'lattice parameter': 5.555,
    'pbe-lp': 5.521,
    'cohesive energy': 1.84,
    'bulk modulus': 18.36,
    'debye temperature': 230.,
    'symbols': 'Ca',
    'latex name': "Ca",
    'structure':'fcc',
    'magmom': None},
'Sr-fcc': {
    'name': 'Sr-fcc',
    'lattice parameter': 6.042,
    'pbe-lp': 6.013,
    'cohesive energy': 1.72,
    'bulk modulus': 12.36,
    'debye temperature': 147.,
    'symbols': 'Sr',
    'latex name': "Sr",
    'structure':'fcc',
    'magmom': None},
'Ba-bcc': {
    'name': 'Ba-bcc',
    'lattice parameter': 5.004,
    'pbe-lp': 5.022,
    'cohesive energy': 1.90,
    'bulk modulus': 9.3,
    'debye temperature': 110.,
    'symbols': 'Ba',
    'latex name': "Ba",
    'structure':'bcc',
    'magmom': None},
'V-bcc': {
    'name': 'V-bcc',
    'lattice parameter': 3.024,
    'pbe-lp': 2.997,
    'cohesive energy': 5.31,
    'bulk modulus': 161.9,
    'debye temperature': 380.,
    'symbols': 'V',
    'latex name': "V",
    'structure':'bcc',
    'magmom': None},
'Nb-bcc': {
    'name': 'Nb-bcc',
    'lattice parameter': 3.293,
    'pbe-lp': 3.310,
    'cohesive energy': 7.57,
    'bulk modulus': 170.2,
    'debye temperature': 275.,
    'symbols': 'Nb',
    'latex name': "Nb",
    'structure':'bcc',
    'magmom': None},
'Ta-bcc': {
    'name': 'Ta-bcc',
    'lattice parameter': 3.299,
    'pbe-lp': 3.347,
    'cohesive energy': 8.10,
    'bulk modulus': 194.,
    'debye temperature': 240.,
    'symbols': 'Ta',
    'latex name': "Ta",
    'structure':'bcc',
    'magmom': None},
'Mo-bcc': {
    'name': 'Mo-bcc',
    'lattice parameter': 3.141,
    'pbe-lp': 3.161,
    'cohesive energy': 6.82,
    'bulk modulus': 272.5,
    'debye temperature': 450.,
    'symbols': 'Mo',
    'latex name': "Mo",
    'structure':'bcc',
    'magmom': None},
'W-bcc': {
    'name': 'W-bcc',
    'lattice parameter': 3.161,
    'pbe-lp': 3.170,
    'cohesive energy': 8.90,
    'bulk modulus': 314.,
    'debye temperature': 400.,
    'symbols': 'W',
    'latex name': "W",
    'structure':'bcc',
    'magmom': None},
'Fe-bcc': {
    'name': 'Fe-bcc',
    'lattice parameter': 2.855,
    'pbe-lp': 2.834,
    'cohesive energy': 4.28,
    'bulk modulus': 168.3,
    'debye temperature': 470.,
    'symbols': 'Fe',
    'latex name': "Fe",
    'structure':'bcc',
    'magmom': 2.22},
'Rh-fcc': {
    'name': 'Rh-fcc',
    'lattice parameter': 3.793,
    'pbe-lp': 3.827,
    'cohesive energy': 5.75,
    'bulk modulus': 269.,
    'debye temperature': 480.,
    'symbols': 'Rh',
    'latex name': "Rh",
    'structure':'fcc',
    'magmom': None},
'Ir-fcc': {
    'name': 'Ir-fcc',
    'lattice parameter': 3.832,
    'pbe-lp': 3.872,
    'cohesive energy': 6.94,
    'bulk modulus': 355.,
    'debye temperature': 420.,
    'symbols': 'Ir',
    'latex name': "Ir",
    'structure':'fcc',
    'magmom': None},
'Ni-fcc': {
    'name': 'Ni-fcc',
    'lattice parameter': 3.509,
    'pbe-lp': 3.517,
    'cohesive energy': 4.44,
    'bulk modulus': 188.,
    'debye temperature': 450.,
    'symbols': 'Ni',
    'latex name': "Ni",
    'structure':'fcc',
    'magmom': 0.64},
'Pd-fcc': {
    'name': 'Pd-fcc',
    'lattice parameter': 3.876,
    'pbe-lp': 3.932,
    'cohesive energy': 3.89,
    'bulk modulus': 195.,
    'debye temperature': 274.,
    'symbols': 'Pd',
    'latex name': "Pd",
    'structure':'fcc',
    'magmom': None},
'Pt-fcc': {
    'name': 'Pt-fcc',
    'lattice parameter': 3.913,
    'pbe-lp': 3.971,
    'cohesive energy': 5.84,
    'bulk modulus': 278.3,
    'debye temperature': 240.,
    'symbols': 'Pt',
    'latex name': "Pt",
    'structure':'fcc',
    'magmom': None},
'Cu-fcc': {
    'name': 'Cu-fcc',
    'lattice parameter': 3.595,
    'pbe-lp': 3.630,
    'cohesive energy': 3.49,
    'bulk modulus': 142.,
    'debye temperature': 343.,
    'symbols': 'Cu',
    'latex name': "Cu",
    'structure':'fcc',
    'magmom': None},
'Ag-fcc': {
    'name': 'Ag-fcc',
    'lattice parameter': 4.063,
    'pbe-lp': 4.150,
    'cohesive energy': 2.95,
    'bulk modulus': 109.,
    'debye temperature': 225.,
    'symbols': 'Ag',
    'latex name': "Ag",
    'structure':'fcc',
    'magmom': None},
'Au-fcc': {
    'name': 'Au-fcc',
    'lattice parameter': 4.061,
    'pbe-lp': 4.147,
    'cohesive energy': 3.81,
    'bulk modulus': 180.,
    'debye temperature': 165.,
    'symbols': 'Au',
    'latex name': "Au",
    'structure':'fcc',
    'magmom': None},
'Al-fcc': {
    'name': 'Al-fcc',
    'lattice parameter': 4.019,
    'pbe-lp': 4.037,
    'cohesive energy': 3.39,
    'bulk modulus': 79.4,
    'debye temperature': 428.,
    'symbols': 'Al',
    'latex name': "Al",
    'structure':'fcc',
    'magmom': None},
'C-diamond': {
    'name': 'C-diamond',
    'lattice parameter': 3.555,
    'pbe-lp': 3.571,
    'cohesive energy': 7.37,
    'bulk modulus': 443.,
    'debye temperature': 2230.,
    'symbols': 'C',
    'latex name': "C (diamond)",
    'structure':'diamond',
    'magmom': None},
'Si-diamond': {
    'name': 'Si-diamond',
    'lattice parameter': 5.422,
    'pbe-lp': 5.468,
    'cohesive energy': 4.63,
    'bulk modulus': 99.2,
    'debye temperature': 645.,
    'symbols': 'Si',
    'latex name': "Si (diamond)",
    'structure':'diamond',
    'magmom': None},
'Ge-diamond': {
    'name': 'Ge-diamond',
    'lattice parameter': 5.644,
    'pbe-lp': 5.764,
    'cohesive energy': 3.85,
    'bulk modulus': 75.8,
    'debye temperature': 374.,
    'latex name': "Ge",
    'symbols': 'Ge',
    'structure':'diamond',
    'magmom': None},
'Sn-diamond': {
    'name': 'Sn-fcc',
    'lattice parameter': 6.476,
    'pbe-lp': 6.659,
    'cohesive energy': 3.14,
    'bulk modulus': 53.1,
    'debye temperature': 200.,
    'symbols': 'Sn',
    'latex name': "Sn",
    'structure':'diamond',
    'magmom': None},
'Pb-fcc': {
    'name': 'Pb-fcc',
    'lattice parameter': 4.912,
    'pbe-lp': 5.040,
    'cohesive energy': 2.03,
    'bulk modulus': 48.8,
    'debye temperature': 105.,
    'symbols': 'Pb',
    'latex name': "Pb",
    'structure':'fcc',
    'magmom': None},
# hcp pure solids
'Cd-hcp': {
    'name': "Cd-hcp",
    'lattice parameter': 2.979,
    'lattice parameter 2': 5.620,
    'revtpss-lp': 2.95057912237,
    'cohesive energy': 1.16,
    'bulk modulus kittel': 46.7, # GPa
    'symbols': 'Cd',
    'latex name': "Cd",
    'structure':'hcp',
    'magmom': None},
'Co-hcp': {
    'name': "Co-hcp",
    'lattice parameter': 2.507,
    'lattice parameter 2': 4.069,
    'revtpss-lp':  2.45918897125,
    'cohesive energy': 4.39,
    'bulk modulus kittel': 191.4, # GPa
    'symbols': 'Co',
    'latex name': "Co",
    'structure':'hcp',
    'magmom': 1.72},
    #'gpaw-conv': [1.e-5, 1.e-6, 1.e-4]},
'Os-hcp': {
    'name': "Os-hcp",
    'lattice parameter': 2.734,
    'lattice parameter 2': 4.392,
    'revtpss-lp': 2.73044397786,
    'cohesive energy': 8.17,
    'bulk modulus kittel': 418., # GPa
    'symbols': 'Os',
    'latex name': "Os",
    'structure':'hcp',
    'magmom': None},
'Ru-hcp': {
    'name': "Ru-hcp",
    'lattice parameter': 2.706,
    'lattice parameter 2': 4.282,
    'revtpss-lp':  2.7191530774,
    'cohesive energy': 6.74,
    'bulk modulus kittel': 320.8, # GPa
    'symbols': 'Ru',
    'latex name': "Ru",
    'structure':'hcp',
    'magmom': None},
'Zn-hcp': {
    'name': "Zn-hcp",
    'lattice parameter': 2.665,
    'lattice parameter 2': 4.947,
    'revtpss-lp': 2.59872461224,
    'cohesive energy': 1.35,
    'bulk modulus kittel': 59.8, # GPa
    'symbols': 'Zn',
    'latex name': "Zn",
    'structure':'hcp',
    'magmom': None},
'Ti-hcp': {
    'name': "Ti-hcp",
    'lattice parameter': 2.951,
    'lattice parameter 2': 4.684,
    'revtpss-lp': 2.92208023815,
    'cohesive energy': 4.85,
    'bulk modulus kittel': 105.1, # GPa
    'symbols': 'Ti',
    'latex name': "Ti",
    'structure':'hcp',
    'magmom': None,
    'gpaw-conv': [1.e-6, 1.e-4, 1.e-6]},
'Zr-hcp': {
    'name': "Zr-hcp",
    'lattice parameter': 3.232,
    'lattice parameter 2': 5.148,
    'revtpss-lp': 3.23504460776,
    'cohesive energy': 6.25,
    'bulk modulus kittel': 83.3, # GPa
    'symbols': 'Zr',
    'latex name': "Zr",
    'structure':'hcp',
    'magmom': None},
'Sc-hcp': {
    'name': "Sc-hcp",
    'lattice parameter': 3.309,
    'lattice parameter 2': 5.268,
    'revtpss-lp': 3.30710158266,
    'cohesive energy': 3.90,
    'bulk modulus kittel': 43.5, # GPa
    'symbols': 'Sc',
    'latex name': "Sc",
    'structure':'hcp',
    'magmom': None},
'Be-hcp': {
    'name': "Be-hcp",
    'lattice parameter': 2.286,
    'lattice parameter 2': 3.585,
    'revtpss-lp': 2.27878331698,
    'cohesive energy': 3.32,
    'bulk modulus kittel': 100.3, # GPa
    'symbols': 'Be',
    'latex name': "Be",
    'structure':'hcp',
    'magmom': None},
'Mg-hcp': {
    'name': "Mg-hcp",
    'lattice parameter': 3.209,
    'lattice parameter 2': 5.211,
    'revtpss-lp': 3.17456515719,
    'cohesive energy': 1.51,
    'bulk modulus kittel': 35.4, # GPa
    'symbols': 'Mg',
    'latex name': "Mg",
    'structure':'hcp',
    'magmom': None},
# compounds
'LiH-b1': {
    'name': 'LiH-b1',
    'lattice parameter': 3.979,
    'pbe-lp': 4.006,
    'cohesive energy': 2.49,
    'debye temperature': None,
    'bulk modulus': None,
    'symbols': 'LiH',
    'latex name': "LiH",
    'structure':'rocksalt',
    'magmom': None},
'LiF-b1': {
    'name': 'LiF-b1',
    'lattice parameter': 3.974,
    'pbe-lp': 4.064,
    'cohesive energy': 4.46,
    'debye temperature': 732.,
    'bulk modulus': 69.8,
    'symbols': 'LiF',
    'latex name': "LiF",
    'structure':'rocksalt',
    'magmom': None},
'LiCl-b1': {
    'name': 'LiCl-b1',
    'lattice parameter': 5.072,
    'pbe-lp': 5.147,
    'cohesive energy': 3.59,
    'debye temperature': 422.,
    'bulk modulus': 35.4,
    'symbols': 'LiCl',
    'latex name': "LiCl",
    'structure':'rocksalt',
    'magmom': None},
'NaF-b1': {
    'name': 'NaF-b1',
    'lattice parameter': 4.570,
    'pbe-lp': 4.700,
    'cohesive energy': 3.97,
    'debye temperature': 492.,
    'bulk modulus': 51.4,
    'symbols': 'NaF',
    'latex name': "NaF",
    'structure':'rocksalt',
    'magmom': None},
'NaCl-b1': {
    'name': 'NaCl-b1',
    'lattice parameter': 5.565,
    'pbe-lp': 5.695,
    'cohesive energy': 3.34,
    'debye temperature': 321.,
    'bulk modulus': 26.6,
    'symbols': 'NaCl',
    'latex name': "NaCl",
    'structure':'rocksalt',
    'magmom': None},
'MgO-b1': {
    'name': 'MgO-b1',
    'lattice parameter': 4.188,
    'pbe-lp': 4.255,
    'cohesive energy': 5.20,
    'debye temperature': 946.,
    'bulk modulus': 165.,
    'symbols': 'MgO',
    'latex name': "MgO",
    'structure':'rocksalt',
    'magmom': None},
'MgS-b1': {
    'name': 'MgS-b1',
    'lattice parameter': 5.188,
    'pbe-lp': 5.228,
    'cohesive energy': 4.07,
    'debye temperature': 650.,
    'bulk modulus': 80.,
    'symbols': 'MgS',
    'latex name': "MgS",
    'structure':'rocksalt',
    'magmom': None},
'CaO-b1': {
    'name': 'CaO-b1',
    'lattice parameter': 4.781,
    'pbe-lp': 4.832,
    'cohesive energy': 5.57,
    'debye temperature': 648.,
    'bulk modulus': 115.,
    'symbols': 'CaO',
    'latex name': "CaO",
    'structure':'rocksalt',
    'magmom': None},
'TiC-b1': {
    'name': 'TiC-b1',
    'lattice parameter': 4.318,
    'pbe-lp': 4.332,
    'cohesive energy': 7.16,
    'debye temperature': 940.,
    'bulk modulus': 233.,
    'symbols': 'TiC',
    'latex name': "TiC",
    'structure':'rocksalt',
    'magmom': None},
'TiN-b1': {
    'name': 'TiN-b1',
    'lattice parameter': 4.226,
    'pbe-lp': 4.247,
    'cohesive energy': 6.69,
    'debye temperature': 757.,
    'bulk modulus': 277.,
    'symbols': 'TiN',
    'latex name': "TiN",
    'structure':'rocksalt',
    'magmom': None},
'ZrC-b1': {
    'name': 'ZrC-b1',
    'lattice parameter': 4.687,
    'pbe-lp': 4.708,
    'cohesive energy': 7.93,
    'debye temperature': 700.,
    'bulk modulus': 207.,
    'symbols': 'ZrC',
    'latex name': "ZrC",
    'structure':'rocksalt',
    'magmom': None},
'ZrN-b1': {
    'name': 'ZrN-b1',
    'lattice parameter': 4.576,
    'pbe-lp': 4.594,
    'cohesive energy': 7.52,
    'debye temperature': 700.,
    'bulk modulus': 240.,
    'symbols': 'ZrN',
    'latex name': "ZrN",
    'structure':'rocksalt',
    'magmom': None},
'VC-b1': {
    'name': 'VC-b1',
    'lattice parameter': 4.148,
    'pbe-lp': 4.154,
    'cohesive energy': 6.94,
    'debye temperature': 971.,
    'bulk modulus': 303.,
    'symbols': 'VC',
    'latex name': "VC",
    'structure':'rocksalt',
    'magmom': None,
    'gpaw-conv': [1.e-6, 1.e-3, 1.e-6]},
'VN-b1': {
    'name': 'VN-b1',
    'lattice parameter': 4.130,
    'pbe-lp': 4.116,
    'cohesive energy': 6.25,
    'debye temperature': 755.,
    'bulk modulus': 268.,
    'symbols': 'VN',
    'latex name': "VN",
    'structure':'rocksalt',
    'magmom': None},
'NbC-b1': {
    'name': 'NbC-b1',
    'lattice parameter': 4.461,
    'pbe-lp': 4.484,
    'cohesive energy': 8.26,
    'debye temperature': 761.,
    'bulk modulus': 300.,
    'symbols': 'NbC',
    'latex name': "NbC",
    'structure':'rocksalt',
    'magmom': None},
'NbN-b1': {
    'name': 'NbN-b1',
    'lattice parameter': 4.383,
    'pbe-lp': 4.422,
    'cohesive energy': 7.50,
    'debye temperature': 730.,
    'bulk modulus': 287.,
    'symbols': 'NbN',
    'latex name': "NbN",
    'structure':'rocksalt',
    'magmom': None},
'FeAl-b2': {
    'name': 'FeAl-b2',
    'lattice parameter': 2.881,
    'pbe-lp': 2.868,
    'cohesive energy': 4.23,
    'debye temperature': 514.,
    'bulk modulus': 136.1,
    'symbols': 'FeAl',
    'latex name': "FeAl",
    'structure':'cesiumchloride',
    'magmom': 0.35},
'CoAl-b2': {
    'name': 'CoAl-b2',
    'lattice parameter': 2.854,
    'pbe-lp': 2.851,
    'cohesive energy': 4.35,
    'debye temperature': 500.,
    'bulk modulus': 162.,
    'symbols': 'CoAl',
    'latex name': "CoAl",
    'structure':'cesiumchloride',
    'magmom': None},
'NiAl-b2': {
    'name': 'NiAl-b2',
    'lattice parameter': 2.881,
    'pbe-lp': 2.892,
    'cohesive energy': 4.46,
    'debye temperature': 402.,
    'bulk modulus': 166.,
    'symbols': 'NiAl',
    'latex name': "NiAl",
    'structure':'cesiumchloride',
    'magmom': None},
'BN-b3': {
    'name': 'BN-b3',
    'lattice parameter': 3.594,
    'pbe-lp': 3.624,
    'cohesive energy': 6.76,
    'debye temperature': 1700.,
    'bulk modulus': 372.3,
    'symbols': 'BN',
    'latex name': "BN",
    'structure':'zincblende',
    'magmom': None},
'BP-b3': {
    'name': 'BP-b3',
    'lattice parameter': 4.527,
    'pbe-lp': 4.548,
    'cohesive energy': 5.14,
    'debye temperature': 985.,
    'bulk modulus': 152.,
    'symbols': 'BP',
    'latex name': "BP",
    'structure':'zincblende',
    'magmom': None},
'BAs-b3': {
    'name': 'BAs-b3',
    'lattice parameter': 4.764,
    'pbe-lp': 4.809,
    'cohesive energy': None,
    'debye temperature': 800.,
    'bulk modulus': 138.,
    'symbols': 'BAs',
    'latex name': "BAs",
    'structure':'zincblende',
    'magmom': None},
'AlN-b3': {
    'name': 'AlN-b3',
    'lattice parameter': 4.368,
    'pbe-lp': 4.402,
    'cohesive energy': 5.85,
    'debye temperature': None,
    'bulk modulus': None,
    'symbols': 'AlN',
    'latex name': "AlN",
    'structure':'zincblende',
    'magmom': None},
'AlP-b3': {
    'name': 'AlP-b3',
    'lattice parameter': 5.450,
    'pbe-lp': 5.504,
    'cohesive energy': 4.32,
    'debye temperature': 588.,
    'bulk modulus': 74.4,
    'symbols': 'AlP',
    'latex name': "AlP",
    'structure':'zincblende',
    'magmom': None},
'AlAs-b3': {
    'name': 'AlAs-b3',
    'lattice parameter': 5.649,
    'pbe-lp': 5.728,
    'cohesive energy': 3.82,
    'debye temperature': 292.,
    'bulk modulus': 74.,
    'symbols': 'AlAs',
    'latex name': "AlAs",
    'structure':'zincblende',
    'magmom': None},
'GaN-b3': {
    'name': 'GaN-b3',
    'lattice parameter': 4.523,
    'pbe-lp': 4.549,
    'cohesive energy': 4.55,
    'debye temperature': 600.,
    'bulk modulus': 195.,
    'symbols': 'GaN',
    'latex name': "GaN",
    'structure':'zincblende',
    'magmom': None},
'GaP-b3': {
    'name': 'GaP-b3',
    'lattice parameter': 5.441,
    'pbe-lp': 5.506,
    'cohesive energy': 3.61,
    'debye temperature': 445.,
    'bulk modulus': 87.4,
    'symbols': 'GaP',
    'latex name': "GaP",
    'structure':'zincblende',
    'magmom': None},
'GaAs-b3': {
    'name': 'GaAs-b3',
    'lattice parameter': 5.641,
    'pbe-lp': 5.751,
    'cohesive energy': 3.34,
    'debye temperature': 344.,
    'bulk modulus': 75.6,
    'symbols': 'GaAs',
    'latex name': "GaAs",
    'structure':'zincblende',
    'magmom': None},
'InP-b3': {
    'name': 'InP-b3',
    'lattice parameter': 5.858,
    'pbe-lp': 5.963,
    'cohesive energy': 3.47,
    'debye temperature': 321.,
    'bulk modulus': 71.,
    'symbols': 'InP',
    'latex name': "InP",
    'structure':'zincblende',
    'magmom': None},
'InAs-b3': {
    'name': 'InAs-b3',
    'lattice parameter': 6.048,
    'pbe-lp': 6.188,
    'cohesive energy': 3.08,
    'debye temperature': 247.,
    'bulk modulus': 58.,
    'symbols': 'InAs',
    'latex name': "InAs",
    'structure':'zincblende',
    'magmom': None},
'SiC-b3': {
    'name': 'SiC-b3',
    'lattice parameter': 4.348,
    'pbe-lp': 4.378,
    'cohesive energy': 6.48,
    'debye temperature': 1232.,
    'bulk modulus': 225.,
    'symbols': 'SiC',
    'latex name': "SiC",
    'structure':'zincblende',
    'magmom': None},
# Pearson
'KBr-b1': {
    'name': 'KBr-b1',
    'lattice parameter': 6.60,
    'revtpss-lp': 6.75894864163,
    'cohesive energy': 3.08,
    'symbols': 'KBr',
    'latex name': "KBr",
    'structure':'rocksalt',
    'magmom': None},
'CaSe-b1': {
    'name': 'CaSe-b1',
    'lattice parameter': 5.80,
    'revtpss-lp': 5.98316194871,
    'cohesive energy': 4.01,
    'symbols': 'CaSe',
    'latex name': "CaSe",
    'structure':'rocksalt',
    'magmom': None},
'SeAs-b1': {
    'name': 'SeAs-b1',
    'lattice parameter': 5.48,
    'revtpss-lp': 5.55199610297,
    'cohesive energy': 2.46, #4.92,
    'symbols': 'SeAs',
    'latex name': "SeAs",
    'structure':'rocksalt',
    'magmom': None},
'RbI-b1': {
    'name': 'RbI-b1',
    'lattice parameter': 7.00,
    'revtpss-lp': None,
    'cohesive energy': 2.71,
    'symbols': 'RbI',
    'latex name': "RbI",
    'structure':'rocksalt',
    'magmom': None},
'LiI-b1': {
    'name': 'LiI-b1',
    'lattice parameter': 6.00,
    'revtpss-lp': 5.98205708599,
    'cohesive energy': 2.78,
    'symbols': 'LiI',
    'latex name': "LiI",
    'structure':'rocksalt',
    'magmom': None},
'CsF-b1': {
    'name': 'CsF-b1',
    'lattice parameter': 6.02,
    'revtpss-lp': 6.21937874326,
    'cohesive energy': 3.66,
    'symbols': 'CsF',
    'latex name': "CsF",
    'structure':'rocksalt',
    'magmom': None},
'CsI-b2': {
    'name': 'CsI-b2',
    'lattice parameter': 4.42,
    'revtpss-lp': 4.6376073499,
    'cohesive energy': 2.75,
    'symbols': 'CsI',
    'latex name': "CsI",
    'structure':'cesiumchloride',
    'magmom': None},
'AgF-b1': {
    'name': 'AgF-b1',
    'lattice parameter': 4.92,
    'revtpss-lp': 4.95954689488,
    'cohesive energy': 2.95,
    'symbols': 'AgF',
    'latex name': "AgF",
    'structure':'rocksalt',
    'magmom': None},
'AgCl-b1': {
    'name': 'AgCl-b1',
    'lattice parameter': 5.56,
    'revtpss-lp': 5.55845581698,
    'cohesive energy': 2.75,
    'symbols': 'AgCl',
    'latex name': "AgCl",
    'structure':'rocksalt',
    'magmom': None},
'AgBr-b1': {
    'name': 'AgBr-b1',
    'lattice parameter': 5.78,
    'revtpss-lp': 5.7885713515,
    'cohesive energy': 2.58,
    'symbols': 'AgBr',
    'latex name': "AgBr",
    'structure':'rocksalt',
    'magmom': None},
'CaS-b1': {
    'name': 'CaS-b1',
    'lattice parameter': 5.70,
    'revtpss-lp': 5.7588414509,
    'cohesive energy': 4.81,
    'symbols': 'CaS',
    'latex name': "CaS",
    'structure':'rocksalt',
    'magmom': None},
'BaO-b1': {
    'name': 'BaO-b1',
    'lattice parameter': 5.52,
    'revtpss-lp': 5.60522427797,
    'cohesive energy': 5.10,
    'symbols': 'BaO',
    'latex name': "BaO",
    'structure':'rocksalt',
    'magmom': None},
'BaSe-b1': {
    'name': 'BaSe-b1',
    'lattice parameter': 6.60,
    'revtpss-lp': 6.67681841117,
    'cohesive energy': 4.03,
    'symbols': 'BaSe',
    'latex name': "BaSe",
    'structure':'rocksalt',
    'magmom': None},
'CdO-b1': {
    'name': 'CdO-b1',
    'lattice parameter': 4.70,
    'revtpss-lp': 4.75039242719,
    'cohesive energy': 3.21,
    'symbols': 'CdO',
    'latex name': "CdO",
    'structure':'rocksalt',
    'magmom': None},
'MnO-b1': {
    'name': 'MnO-b1',
    'lattice parameter': 4.44,
    'revtpss-lp': 4.4520326675,
    'cohesive energy': 4.75,
    'symbols': 'MnO',
    'latex name': "MnO",
    'structure':'rocksalt',
    'magmom': 2.4},
'MnS-b1': {
    'name': 'MnS-b1',
    'lattice parameter': 5.22,
    'revtpss-lp': 5.13039706051,
    'cohesive energy': 4.01,
    'symbols': 'MnS',
    'latex name': "MnS",
    'structure':'rocksalt',
    'magmom': 2.4},
# Haglund
'ScC-b1': {
    'name': 'ScC-b1',
    'lattice parameter': 4.72,
    'revtpss-lp': 4.69537210811,
    'cohesive energy': 6.37,
    'symbols': 'ScC',
    'latex name': "ScC",
    'structure':'rocksalt',
    'magmom': None},
'CrC-b1': { # slow gpaw convergence
    'name': 'CrC-b1',
    'lattice parameter': 4.12,
    'revtpss-lp': 4.12680477339,
    'cohesive energy': 5.80,
    'symbols': 'CrC',
    'latex name': "CrC",
    'structure':'rocksalt',
    'magmom': 0.6},
    #'gpaw-conv': [1.e-5, 1.e-7, 1.e-4]},
'MnC-b1': {
    'name': 'MnC-b1',
    'lattice parameter': 4.12,
    'revtpss-lp': 4.11651335966,
    'cohesive energy': 5.14,
    'symbols': 'MnC',
    'latex name': "MnC",
    'structure':'rocksalt',
    'magmom': 1.2},
    #'gpaw-conv': [1.e-5, 1.e-6, 1.e-4]},
'FeC-b1': {
    'name': 'FeC-b1',
    'lattice parameter': 4.09,
    'revtpss-lp': 3.98592429783,
    'cohesive energy': 5.67,
    'symbols': 'FeC',
    'latex name': "FeC",
    'structure':'rocksalt',
    'magmom': None},
'CoC-b1': {
    'name': 'CoC-b1',
    'lattice parameter': 4.05,
    'revtpss-lp': 3.98558621985,
    'cohesive energy': 5.69,
    'symbols': 'CoC',
    'latex name': "CoC",
    'structure':'rocksalt',
    'magmom': None},
'NiC-b1': {
    'name': 'NiC-b1',
    'lattice parameter': 3.99,
    'revtpss-lp': 4.03667215219,
    'cohesive energy': 5.65,
    'symbols': 'NiC',
    'latex name': "NiC",
    'structure':'rocksalt',
    'magmom': None},
'ScN-b1': {
    'name': 'ScN-b1',
    'lattice parameter': 4.51,
    'revtpss-lp': 4.52753210905,
    'cohesive energy': 6.72,
    'symbols': 'ScN',
    'latex name': "ScN",
    'structure':'rocksalt',
    'magmom': None},
'CrN-b1': {
    'name': 'CrN-b1',
    'lattice parameter': 4.15,
    'revtpss-lp': 4.18609030986,
    'cohesive energy': 5.14,
    'symbols': 'CrN',
    'latex name': "CrN",
    'structure':'rocksalt',
    'magmom': 1.3},
'MnN-b1': {
    'name': 'MnN-b1',
    'lattice parameter': 4.20,
    'revtpss-lp': 4.15891205836,
    'cohesive energy': 4.08,
    'symbols': 'MnN',
    'latex name': "MnN",
    'structure':'rocksalt',
    'magmom': 1.6},
    #'gpaw-conv': [1.e-5, 1.e-7, 1.e-4]},
'FeN-b1': {
    'name': 'FeN-b1',
    'lattice parameter': 4.13,
    'revtpss-lp': 4.12,
    'cohesive energy': 4.59,
    'symbols': 'FeN',
    'latex name': "FeN",
    'structure':'rocksalt',
    'magmom': 1.3},
    #'gpaw-conv': [1.e-5, 1.e-4, 1.e-7]},
'CoN-b1': {
    'name': 'CoN-b1',
    'lattice parameter': 4.10,
    'revtpss-lp': 3.99392851055,
    'cohesive energy': 4.53,
    'symbols': 'CoN',
    'latex name': "CoN",
    'structure':'rocksalt',
    'magmom': None},
'NiN-b1': {
    'name': 'NiN-b1',
    'lattice parameter': 4.10,
    'revtpss-lp': 4.03914234139,
    'cohesive energy': 4.48,
    'symbols': 'NiN',
    'latex name': "NiN",
    'structure':'rocksalt',
    'magmom': None},
'MoC-b1': {
    'name': 'MoC-b1',
    'lattice parameter': 4.278,
    'revtpss-lp': 4.39395801553,
    'cohesive energy': 7.22,
    'symbols': 'MoC',
    'latex name': "MoC",
    'structure':'rocksalt',
    'magmom': None},
'RuC-b1': {
    'name': 'RuC-b1',
    'lattice parameter': 4.129,
    'revtpss-lp': 4.33092551761,
    'cohesive energy': 6.73,
    'symbols': 'RuC',
    'latex name': "RuC",
    'structure':'rocksalt',
    'magmom': None},
'RhC-b1': {
    'name': 'RhC-b1',
    'lattice parameter': 4.145,
    'revtpss-lp': 4.35180356284,
    'cohesive energy': 6.23,
    'symbols': 'RhC',
    'latex name': "RhC",
    'structure':'rocksalt',
    'magmom': None},
'PdC-b1': {
    'name': 'PdC-b1',
    'lattice parameter': 4.221,
    'revtpss-lp': 4.4101528149,
    'cohesive energy': 5.36,
    'symbols': 'PdC',
    'latex name': "PdC",
    'structure':'rocksalt',
    'magmom': None},
'MoN-b1': {
    'name': 'MoN-b1',
    'lattice parameter': 4.214,
    'revtpss-lp': 4.37920656361,
    'cohesive energy': 6.20,
    'symbols': 'MoN',
    'latex name': "MoN",
    'structure':'rocksalt',
    'magmom': None},
'RuN-b1': {
    'name': 'RuN-b1',
    'lattice parameter': 4.058,
    'revtpss-lp': 4.34894810722,
    'cohesive energy': 5.02,
    'symbols': 'RuN',
    'latex name': "RuN",
    'structure':'rocksalt',
    'magmom': None},
'RhN-b1': {
    'name': 'RhN-b1',
    'lattice parameter': 4.082,
    'revtpss-lp': 4.35633367368,
    'cohesive energy': 4.78,
    'symbols': 'RhN',
    'latex name': "RhN",
    'structure':'rocksalt',
    'magmom': None},
'PdN-b1': {
    'name': 'PdN-b1',
    'lattice parameter': 4.145,
    'revtpss-lp': 4.41700841872,
    'cohesive energy': 4.03,
    'symbols': 'PdN',
    'latex name': "PdN",
    'structure':'rocksalt',
    'magmom': None},
'LaC-b1': {
    'name': 'LaC-b1',
    'lattice parameter': 5.429,
    'revtpss-lp': 5.50510598304,
    'cohesive energy': 5.74,
    'symbols': 'LaC',
    'latex name': "LaC",
    'structure':'rocksalt',
    'magmom': None},
'TaC-b1': {
    'name': 'TaC-b1',
    'lattice parameter': 4.457,
    'revtpss-lp': 4.48669679096,
    'cohesive energy': 8.56,
    'symbols': 'TaC',
    'latex name': "TaC",
    'structure':'rocksalt',
    'magmom': None},
'WC-b1': {
    'name': 'WC-b1',
    'lattice parameter': 4.266,
    'revtpss-lp': 4.40260619124,
    'cohesive energy': 8.25,
    'symbols': 'WC',
    'latex name': "WC",
    'structure':'rocksalt',
    'magmom': None},
'OsC-b1': {
    'name': 'OsC-b1',
    'lattice parameter': 4.176,
    'revtpss-lp': 4.35787411212,
    'cohesive energy': 7.36,
    'symbols': 'OsC',
    'latex name': "OsC",
    'structure':'rocksalt',
    'magmom': None},
'IrC-b1': {
    'name': 'IrC-b1',
    'lattice parameter': 4.129,
    'revtpss-lp': 4.39419683105,
    'cohesive energy': 6.84,
    'symbols': 'IrC',
    'latex name': "IrC",
    'structure':'rocksalt',
    'magmom': None},
'PtC-b1': {
    'name': 'PtC-b1',
    'lattice parameter': 4.206,
    'revtpss-lp': 4.45741920763,
    'cohesive energy': 6.34,
    'symbols': 'PtC',
    'latex name': "PtC",
    'structure':'rocksalt',
    'magmom': None},
'LaN-b1': {
    'name': 'LaN-b1',
    'lattice parameter': 5.305,
    'revtpss-lp': 5.35277831785,
    'cohesive energy': 6.27,
    'symbols': 'LaN',
    'latex name': "LaN",
    'structure':'rocksalt',
    'magmom': None},
'TaN-b1': {
    'name': 'TaN-b1',
    'lattice parameter': 4.340,
    'revtpss-lp': 4.44437817766,
    'cohesive energy': 7.63,
    'symbols': 'TaN',
    'latex name': "TaN",
    'structure':'rocksalt',
    'magmom': None},
'WN-b1': {
    'name': 'WN-b1',
    'lattice parameter': 4.202,
    'revtpss-lp': 4.39478109563,
    'cohesive energy': 7.01,
    'symbols': 'WN',
    'latex name': "WN",
    'structure':'rocksalt',
    'magmom': None},
'OsN-b1': {
    'name': 'OsN-b1',
    'lattice parameter': 4.058,
    'revtpss-lp': 4.37261212422,
    'cohesive energy': 5.61,
    'symbols': 'OsN',
    'latex name': "OsN",
    'structure':'rocksalt',
    'magmom': None},
'IrN-b1': {
    'name': 'IrN-b1',
    'lattice parameter': 4.074,
    'revtpss-lp': 4.40354481322,
    'cohesive energy': 5.13,
    'symbols': 'IrN',
    'latex name': "IrN",
    'structure':'rocksalt',
    'magmom': None},
'PtN-b1': {
    'name': 'PtN-b1',
    'lattice parameter': 4.137,
    'revtpss-lp': 4.48127067537,
    'cohesive energy': 4.63,
    'symbols': 'PtN',
    'latex name': "PtN",
    'structure':'rocksalt',
    'magmom': None},
}
